# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/neoscr',
                'doc_host': 'https://datarisk-io.github.io',
                'git_url': 'https://github.com/datarisk-io/neoscr',
                'lib_path': 'neoscr'},
  'syms': { 'neoscr.cli': { 'neoscr.cli.build_package': ('cli.html#build_package', 'neoscr/cli.py'),
                            'neoscr.cli.consulta_scr': ('cli.html#consulta_scr', 'neoscr/cli.py'),
                            'neoscr.cli.install_package': ('cli.html#install_package', 'neoscr/cli.py')},
            'neoscr.core': { 'neoscr.core.ConsultaSCR': ('core.html#consultascr', 'neoscr/core.py'),
                             'neoscr.core.ConsultaSCR.__init__': ('core.html#consultascr.__init__', 'neoscr/core.py'),
                             'neoscr.core.ConsultaSCR._get_resumo_cliente_traduzido': ( 'core.html#consultascr._get_resumo_cliente_traduzido',
                                                                                        'neoscr/core.py'),
                             'neoscr.core.ConsultaSCR._get_resumo_lista_das_operacoes': ( 'core.html#consultascr._get_resumo_lista_das_operacoes',
                                                                                          'neoscr/core.py'),
                             'neoscr.core.ConsultaSCR._get_resumo_modalidade': ( 'core.html#consultascr._get_resumo_modalidade',
                                                                                 'neoscr/core.py'),
                             'neoscr.core.ConsultaSCR._request': ('core.html#consultascr._request', 'neoscr/core.py'),
                             'neoscr.core.ConsultaSCR.get_cnpj_data': ('core.html#consultascr.get_cnpj_data', 'neoscr/core.py'),
                             'neoscr.core.ConsultaSCR.get_cpf_data': ('core.html#consultascr.get_cpf_data', 'neoscr/core.py')},
            'neoscr.utils': { 'neoscr.utils.fill_zeros': ('utils.html#fill_zeros', 'neoscr/utils.py'),
                              'neoscr.utils.format_cnpj': ('utils.html#format_cnpj', 'neoscr/utils.py'),
                              'neoscr.utils.format_cpf': ('utils.html#format_cpf', 'neoscr/utils.py'),
                              'neoscr.utils.let_only_digits': ('utils.html#let_only_digits', 'neoscr/utils.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['ConsultaSCR']

# %% ../nbs/00_core.ipynb 3
import os
import json
import requests
import pandas as pd
from typing import Tuple
from pathlib import Path
from datetime import datetime

from .utils import let_only_digits
from .utils import codigos_vencimentos

# %% ../nbs/00_core.ipynb 4
class ConsultaSCR:
    def __init__(self, 
                 user: str = None,     # SCR user
                 password: str = None, # SCR password
                 code: str = None,     # SCR code
                 api_key: str = None  # SCR api key
    ):
        "Class to query SCR"
        self.user = os.environ['SCR_USER'] if user is None else user
        self.password = os.environ['SCR_PASSWORD'] if password is None else password
        self.code = os.environ['SCR_CODE'] if code is None else code
        self.key = os.environ['SCR_API_KEY'] if api_key is None else api_key
        self.headers = {'Content-Type': 'application/json'}
        self.url = "https://bmpdigital.moneyp.com.br/api/BMPDigital/ConsultaSCR"
        self.params = {
            "auth": {
                "Usuario": self.user,
                "Senha": self.password,
                "CodigoParametro": self.code,
                "Chave": self.key
            },
            "consulta": {
                "Documento": "",
                "DataBaseMes": "",
                "DataBaseAno": ""
            }
        }

        self.path_to_store_responses = Path.home() / '.neoscr'
        self.path_to_store_responses.mkdir(parents=True, exist_ok=True)
        
    def _request(self, 
                 doc: str,    # CPF or CNPJ
                 ano: int,    # Year to be consulted. Format: YYYY
                 mes: int     # Month to be consulted. Format: MM
    ) -> requests.models.Response:
        "Make a request to the SCR API"

        ano = str(ano)
        mes = f"{mes:02d}"
        self.params["consulta"]["Documento"] = doc
        self.params["consulta"]["DataBaseAno"] = ano
        self.params["consulta"]["DataBaseMes"] = mes

        # if the request was already made, just load the file from the local home storage folder .neoscr
        response_filepath = self.path_to_store_responses / f"{let_only_digits(doc)}_{ano}_{mes}.json"
        if response_filepath.exists():
            print(f"File already exist, loading it from {response_filepath}")
            with open(response_filepath) as json_file:
                response_json = json.load(json_file)
        else:
            print(f"Making a request to SCR api ...")
            response = requests.request("POST", self.url, data=json.dumps(self.params), headers=self.headers)
            response_json = response.json()
            print(f"Saving the response to the ~/.neoscr folder ...")
            with open(response_filepath, 'w') as file:
                json.dump(response_json, file)
                print('Response saved successfully.')
        return response_json

    def _get_resumo_cliente_traduzido(self, 
                                      response_json: dict, # Response from the SCR API 
                                      **kwargs: dict       # Keyword arguments
    ) -> pd.DataFrame:
        "Return a DataFrame with the translated client summary"

        df = pd.DataFrame(response_json.get("ResumoDoClienteTraduzido"), columns=response_json.get("ResumoDoClienteTraduzido").keys(), index = [0])
        df.insert(0, "doc", kwargs.get("doc"))
        df.insert(1, "data_consulta", kwargs.get("data_consulta"))
        df.insert(2, "mes_referencia", kwargs.get("mes_referencia"))
        df.insert(3, "msg", response_json.get("MensagemOperador"))
        return df


    def _get_resumo_modalidade(self, 
                               response_json: dict, # Response from the SCR API 
                               **kwargs: dict       # Keyword arguments
    ) -> pd.DataFrame:
        "Return a DataFrame with the modalities client summary"

        df = pd.DataFrame(response_json.get("ResumoModalidade"))
        df.insert(0, "doc", kwargs.get("doc"))
        df.insert(1, "data_consulta", kwargs.get("data_consulta"))
        df.insert(2, "mes_referencia", kwargs.get("mes_referencia"))
        return df


    def _get_resumo_lista_das_operacoes(self,
                                        response_json: dict, # Response from the SCR API 
                                        **kwargs: dict       # Keyword arguments
    ) -> pd.DataFrame:
        df_aux = pd.DataFrame(response_json.get('ResumoDoCliente').get('ListaDeResumoDasOperacoes'))
        df_aux_exploded = df_aux.explode('ListaDeVencimentos').reset_index(drop=True)
        df = pd.concat([
                df_aux_exploded[['Modalidade', 'VariacaoCambial']],
                df_aux_exploded['ListaDeVencimentos'].apply(pd.Series)
            ], axis=1)
        df.insert(0, "doc", kwargs.get("doc"))
        df.insert(1, "data_consulta", kwargs.get("data_consulta"))
        df.insert(2, "mes_referencia", kwargs.get("mes_referencia"))
        df = pd.merge(df, codigos_vencimentos, on='CodigoVencimento', how='left').filter([
                                                                                        'doc',
                                                                                        'data_consulta',
                                                                                        'mes_referencia',
                                                                                        'Modalidade',
                                                                                        'VariacaoCambial',
                                                                                        'CodigoVencimento',
                                                                                        'Descrição',
                                                                                        'ValorVencimento',
                                                                                        'ValorVencimentoSpecified'])

        return df


    def get_cpf_data(self, 
                     cpf: str,      # CPF
                     ano: int,      # Year to be consulted. Ex: 2022
                     mes: int       # Month to be consulted. Ex: 8, 10
    ) -> Tuple[pd.DataFrame, pd.DataFrame]:
        "Query CPFs and return a tuple of DataFrames: (df_cpf_resumo_cliente_traduzido, df_cpf_resumo_modalidade)"
        
        data_consulta = datetime.today().strftime("%d/%m/%Y")
        response_json = self._request(cpf, ano, mes)
        if response_json.get("Erro"):
            raise Exception(response_json.get("Msg").strip() + f": {cpf}")
        if response_json.get("MensagemOperador") == '(50) - Cliente não possui dados no SCR na data-base.':
            print(f"Cliente não possui dados no SCR na data-base: {cpf}")
            return pd.DataFrame(), pd.DataFrame(), pd.DataFrame()
        
        df_traduzido = self._get_resumo_cliente_traduzido(response_json, doc=cpf, data_consulta=data_consulta, mes_referencia=f"{ano}{mes:02d}")
        df_modalidade = self._get_resumo_modalidade(response_json, doc=cpf, data_consulta=data_consulta, mes_referencia=f"{ano}{mes:02d}")
        df_resumo_lista_das_operacoes = self._get_resumo_lista_das_operacoes(response_json, doc=cpf, data_consulta=data_consulta, mes_referencia=f"{ano}{mes:02d}")
        
        return df_traduzido, df_modalidade, df_resumo_lista_das_operacoes

    
    def get_cnpj_data(self, 
                      cnpj: str,     # CNPJ
                      ano: int,      # Year to be consulted. Ex: 2022
                      mes: int       # Month to be consulted. Ex: 8, 10
    ) -> Tuple[pd.DataFrame, pd.DataFrame]:
        "Query CNPJs and return a tuple of DataFrames: (df_cnpj_resumo_cliente_traduzido, df_cnpj_resumo_modalidade)"
        data_consulta = datetime.today().strftime("%d/%m/%Y")
        response_json = self._request(cnpj, ano, mes)
        if response_json.get("Erro"):
            raise Exception(response_json.get("Msg").strip() + f": {cnpj}")
        if response_json.get("MensagemOperador") == '(50) - Cliente não possui dados no SCR na data-base.':
            print(f"Cliente não possui dados no SCR na data-base: {cnpj}")
            return pd.DataFrame(), pd.DataFrame(), pd.DataFrame()
        
        df_traduzido = self._get_resumo_cliente_traduzido(response_json, doc=cnpj, data_consulta=data_consulta, mes_referencia=f"{ano}{mes:02d}")
        df_modalidade = self._get_resumo_modalidade(response_json, doc=cnpj, data_consulta=data_consulta, mes_referencia=f"{ano}{mes:02d}")
        df_resumo_lista_das_operacoes = self._get_resumo_lista_das_operacoes(response_json, doc=cnpj, data_consulta=data_consulta, mes_referencia=f"{ano}{mes:02d}")

        return df_traduzido, df_modalidade, df_resumo_lista_das_operacoes



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/02_utils.ipynb.

# %% auto 0
__all__ = ['codigos_vencimentos', 'let_only_digits', 'format_cnpj', 'format_cpf', 'fill_zeros']

# %% ../nbs/02_utils.ipynb 2
import pandas as pd

# %% ../nbs/02_utils.ipynb 3
def let_only_digits(doc: str) -> str:
    """Remove all non-digit characters from a string"""
    return ''.join(filter(str.isdigit, doc))

# %% ../nbs/02_utils.ipynb 7
def format_cnpj(cnpj: str) -> str:
    """Format a CNPJ string"""
    return cnpj[:2] + '.' + cnpj[2:5] + '.' + cnpj[5:8] + '/' + cnpj[8:12] + '-' + cnpj[12:]

# %% ../nbs/02_utils.ipynb 9
def format_cpf(cpf: str) -> str:
    """Format a CPF string"""
    return cpf[:3] + '.' + cpf[3:6] + '.' + cpf[6:9] + '-' + cpf[9:]

# %% ../nbs/02_utils.ipynb 11
def fill_zeros(doc: str, n: int) -> str:
    """Fill a string with zeros to the left with n characters"""
    return doc.zfill(n)

# %% ../nbs/02_utils.ipynb 14
codigos_vencimentos = pd.DataFrame([{'Descrição': 'Limite de crédito com vencimento até 360 dias ',
  'CodigoVencimento': 'v20'},
 {'Descrição': 'Limite de crédito com vencimento acima de 360 dias ',
  'CodigoVencimento': 'v40'},
 {'Descrição': 'Créditos a liberar até 360 dias ', 'CodigoVencimento': 'v60'},
 {'Descrição': 'Créditos a liberar acima de 360 dias ',
  'CodigoVencimento': 'v80'},
 {'Descrição': 'Créditos a vencer até 30 dias', 'CodigoVencimento': 'v110'},
 {'Descrição': 'Créditos a vencer de 31 a 60 dias',
  'CodigoVencimento': 'v120'},
 {'Descrição': 'Créditos a vencer de 61 a 90 dias',
  'CodigoVencimento': 'v130'},
 {'Descrição': 'Créditos a vencer de 91 a 180 dias',
  'CodigoVencimento': 'v140'},
 {'Descrição': 'Créditos a vencer de 181 a 360 dias ',
  'CodigoVencimento': 'v150'},
 {'Descrição': 'Créditos a vencer de 361 a 720 dias',
  'CodigoVencimento': 'v160'},
 {'Descrição': 'Créditos a vencer de 721 a 1080 dias',
  'CodigoVencimento': 'v165'},
 {'Descrição': 'Créditos a vencer de 1081 a 1440 dias',
  'CodigoVencimento': 'v170'},
 {'Descrição': 'Créditos a vencer de 1441 a 1800 dias',
  'CodigoVencimento': 'v175'},
 {'Descrição': 'Créditos a vencer de 1801 a 5400 dias',
  'CodigoVencimento': 'v180'},
 {'Descrição': 'Créditos a vencer acima de 5400 dias',
  'CodigoVencimento': 'v190'},
 {'Descrição': 'Créditos a vencer com prazo indeterminado',
  'CodigoVencimento': 'v199'},
 {'Descrição': 'Créditos vencidos de 1 a 14 dias', 'CodigoVencimento': 'v205'},
 {'Descrição': 'Créditos vencidos de 15 a 30 dias',
  'CodigoVencimento': 'v210'},
 {'Descrição': 'Créditos vencidos de 31 a 60 dias',
  'CodigoVencimento': 'v220'},
 {'Descrição': 'Créditos vencidos de 61 a 90 dias',
  'CodigoVencimento': 'v230'},
 {'Descrição': 'Créditos vencidos de 91 a 120 dias',
  'CodigoVencimento': 'v240'},
 {'Descrição': 'Créditos vencidos de 121 a 150 dias',
  'CodigoVencimento': 'v245'},
 {'Descrição': 'Créditos vencidos de 151 a 180 dias',
  'CodigoVencimento': 'v250'},
 {'Descrição': 'Créditos vencidos de 181 a 240 dias',
  'CodigoVencimento': 'v255'},
 {'Descrição': 'Créditos vencidos de 241 a 300 dias',
  'CodigoVencimento': 'v260'},
 {'Descrição': 'Créditos vencidos de 301 a 360 dias',
  'CodigoVencimento': 'v270'},
 {'Descrição': 'Créditos vencidos de 361 a 540 dias',
  'CodigoVencimento': 'v280'},
 {'Descrição': 'Créditos vencidos acima de 540 dias',
  'CodigoVencimento': 'v290'},
 {'Descrição': 'Créditos baixados como prejuízo até 12 meses',
  'CodigoVencimento': 'v310'},
 {'Descrição': 'Créditos baixados como prejuízo há mais de 12m e até 48 meses',
  'CodigoVencimento': 'v320'},
 {'Descrição': 'Créditos baixados como prejuízo há mais de 48 meses',
  'CodigoVencimento': 'v330'}])



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_cli.ipynb.

# %% auto 0
__all__ = ['consulta_scr', 'install_package', 'build_package']

# %% ../nbs/01_cli.ipynb 3
import os
from .core import ConsultaSCR
from fastcore.script import call_parse

# %% ../nbs/01_cli.ipynb 4
@call_parse
def consulta_scr(user:str,     # SCR user
                 password:str, # SCR password
                 code:str,     # SCR code
                 api_key:str,  # SCR API KEY
                 doc:str,      # CNPJs ou CPFs. Deve ser no formato 00.000.000/0000-00 ou 000.000.000-00
                 anomes:str,   # Ano e mês da consulta. Deve ser no formato MM_YYYY.
                 path:str="./scr_data", # Caminho para salvar os arquivos

    ):
    """
    Consulta o SCR e salva os dados em arquivos CSV.
    """

    scr = ConsultaSCR(
        user=user,
        password=password,
        code=code,
        api_key=api_key
    )

    if not os.path.exists(path):
        os.makedirs(path)

    if '/' in doc:
        df_cnpj_traduzido, df_cnpj_modalidade = scr.get_cnpj_data(doc, anomes)
        doc_only_digits = ''.join(filter(str.isdigit, doc))
        df_cnpj_traduzido.to_csv(f'{path}/scr_data_traduzido_{doc_only_digits}.csv', index=False, sep=";")
        df_cnpj_modalidade.to_csv(f'{path}/scr_data_modalidade_{doc_only_digits}.csv', index=False, sep=";")
    else:
        df_cpf_traduzido, df_cpf_modalidade = scr.get_cpf_data(doc, anomes)
        doc_only_digits = ''.join(filter(str.isdigit, doc))
        df_cpf_traduzido.to_csv(f'{path}/scr_data_traduzido_{doc_only_digits}.csv', index=False, sep=";")
        df_cpf_modalidade.to_csv(f'{path}/scr_data_modalidade_{doc_only_digits}.csv', index=False, sep=";")

    return True

# %% ../nbs/01_cli.ipynb 9
@call_parse
def install_package():
    "Install the package"
    os.system("pip install -e '.[dev]'")

# %% ../nbs/01_cli.ipynb 10
@call_parse
def build_package():
    "Build the package and generate the .whl file"
    os.system("python setup.py bdist_wheel")
